<?php

/*  general.php:  Some various helper functions
 *
    Copyright (C) 2002,2003,2004  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *$Id: general.php,v 1.255 2005/05/11 16:04:35 sailfrog Exp $
*/

/**
 *
 *  Convert UTF-8 text to HTML entities for display
 *
 *  @param  $input      string      text to convert to HTML entities 
 *  @return             string      converted text
 */
function hm_utf8_to_html($input) {
    $output = '';
    $index = 0;
    while ($index < strlen($input)) {
        $char = ord($input[$index]);
        switch (true) {
            case $char < 0x80:
                $output .= $input[$index];
                $index += 1;
                break;
            case $char < 0xE0:
                $output .= '&#'.((($char % 0x20) * 0x40) +
                          (ord($input[$index + 1]) % 0x40)).';';
                $index += 2;
                break;
            case $char < 0xF0:
                $output .= '&#'.((($char % 0x10) * 0x1000) + 
                           ((ord($input[$index + 1]) % 0x40) * 0x40) + 
                           (ord($input[$index + 2]) % 0x40)).';';
                $index += 3;
                break;
            case $char < 0xF8:
                $output .= '&#'.((($char % 0x08) * 0x40000) + 
                           ((ord($input[$index + 1]) % 0x40) * 0x1000) +
                           ((ord($input[$index + 2]) % 0x40) * 0x40) +
                           (ord($input[$index + 3]) % 0x40)).';';
                $index += 4;
                break;
            case $char < 0xFC:
                $output .= '&#'.((($char % 0x04) * 0x1000000) + 
                           ((ord($input[$index + 1]) % 0x40) * 0x40000) +
                           ((ord($input[$index + 2]) % 0x40) * 0x1000) +
                           ((ord($input[$index + 3]) % 0x40) * 0x40) +
                           (ord($input[$index + 4]) % 0x40)).';';
                $index += 5;
                break;
            default:
                $output .= '&#'.((($char % 0x02) * 0x40000000) +
                           ((ord($input[$index + 1]) % 0x40) * 0x1000000) +
                           ((ord($input[$index + 2]) % 0x40) * 0x40000) +
                           ((ord($input[$index + 3]) % 0x40) * 0x1000) +
                           ((ord($input[$index + 4]) % 0x40) * 0x40) +
                           (ord($input[$index + 4]) % 0x40)).';';
                $index += 6;
                break;
        }
    }
    return $output;
} 

/**
 *
 * Parse an address field into valid email addresses
 *
 *  @param  $string     string      address field string
 *  @param  $get_name   boolean     if true get names for each address as well
 *  @return             array       actual email addresses
 */
function hm_parse_address ($string, $get_names=false) {
    $parts = array();
    $addy = array();
    $string = str_replace(',', ', ', $string);
    $string = stripslashes($string);

    /* get rid of any comments in parens, even if nested */
    if (strpos($string, '(') !== false) {
        $max = strlen($string);
        $line = '';
        $level = 0;
        for ($i=0;$i<$max;$i++) {
            if ($string{$i} == '(') {
                $level++;
            }
            if ($level == 0) {
                $line .= $string{$i};
            }
            if ($string{$i} == ')') {
                $level--;
            }
        }
        /* if we have mis-matched parens parse the whole bugger anyway */
        if (!$line) {
            $line = $string;
        }
    }
    else {
        $line = $string;
    }
    /* break up the line, grouping any quoted stuff */
    $line = trim($line);
    while ($line) {
        $chunk = false;
        switch ($line{0}) {
            case '"':
                if (preg_match("/^(\"[^\"\\\]*(?:\\\.[^\"\\\]*)*\")/",
                    $line, $matches)) {
                    $chunk = $matches[1];
                }
                $line = substr($line, strlen($chunk));
                break;
            case ' ':
                $line = substr($line, 1);
                break;
            default:
                if (strpos($line, ' ') !== false) {
                    $chunk = substr($line, 0, strpos($line, ' '));
                    $line = substr($line, strlen($chunk));
                }
                else {
                    $chunk = $line;
                    $line = false;
                }
                break;
        }
        if (is_string($chunk)) {
            $parts[] = trim($chunk);
        }
    }
    /* loop through the parts and get valid addys, whilst stripping
       any crap we don't want */
    $i = 0;
    foreach ($parts as $part) {
        if ($i > 0 && $part{0} == '@') {
            $addy[] = preg_replace("/((^&lt;|^<)|(&gt$|>$))/", '',
                      rtrim($parts[$i - 1].$part, ',;'));
        }
        elseif ($part{0} != '"' && preg_match("/[^\s]+@[^\s]{3,}/", $part)) {
             $addy[] = preg_replace("/((^&lt;|^<)|(&gt$|>$))/", '', rtrim($part, ',;'));
        }
        $i++;
    }
    if (empty($addy) && $string != '') {
        if (strpos($string, ',')) {
            $temp_addy = explode(',', $string);
            foreach ($temp_addy as $temp) {
                if (!preg_match("/(\(|\)|\"|\')/", $temp)) {
                    $addy[] = trim($temp);
                }
            }
        }
        else {
            $addy[] = $string;
        }
    }
    return $addy;
}

/**
 *
 *  Reduce saved folder sort data to keep session from getting bloated
 *  @param      $str         array       translation strings
 *  @param      $page        string      current page name
 *  @param      $mailbox     string      mailbox name
 *  @return                  void 
 *
 */
function hm_manage_cache_size ($str, $page, $mailbox) {
    $max = 6000;
    $min = 3000;
    if (isset($_SESSION['config']['folder_cache_max']) &&
        is_numeric($_SESSION['config']['folder_cache_max']) &&
        $_SESSION['config']['folder_cache_max'] > 0) {
        $max = $_SESSION['config']['folder_cache_max'];
    }
    if (isset($_SESSION['config']['folder_cache_min']) &&
        is_numeric($_SESSION['config']['folder_cache_min']) &&
        $_SESSION['config']['folder_cache_max'] > 0 &&
        $_SESSION['config']['folder_cache_min'] < $max) {
        $min = $_SESSION['config']['folder_cache_min'];
    }
    $folder_cache = array();
    $size = 0;
    $folders_count = array();
    $alt_size = 0;
    if ($page != $str[26] && $page != $str[220]) {
        $mailbox = false;
    }
    if (isset($_SESSION['sort_data'])) {
        $folders_count = count($_SESSION['sort_data']);
        foreach($_SESSION['sort_data'] as $key => $array) {
            if ($key != $mailbox && isset($array[2])) {
                $folder_size = count($array[2]);
                $size += $folder_size;
                $folder_cache[$key] = $folder_size;
            }
            else {
                if (isset($array[2])) {
                    $alt_size += count($array[2]);
                }
            }
        }
        $size += $alt_size;
        if ($size > $max) {
            arsort($folder_cache, SORT_NUMERIC);
            foreach ($folder_cache as $name => $fsize) {
                unset($_SESSION['sort_data'][$name]);
                $folders_count--;
                $size -= $fsize;
                if ($size < $min) {
                    break;
                }
            }
        }
    }
    $_SESSION['cached_folders'] = $folders_count;
    $_SESSION['cache_size'] = $size;
}
    
/**
 *
 * Translate "special folder names"
 *  @param      $folder     string      mailbox name
 *  @return                 string      translated name or unchanged folder name
 *
 */
function hm_translate_special_folders($folder) {
    $trans = $_SESSION['str']['folders'];
    if (isset($trans[$folder])) {
        return $trans[$folder];
    }
    else {
        return $folder;
    }
}

/**
 * Set the current domain and mangle the username if need be
 *
 *  @param  $config     array       config values
 *  @param  $user       string      login username 
 *  @return             array       correct username and domain
 *
 */
function hm_get_domain_and_username ($config, $user) {
    $domain = false;
    if (isset($config['append_name']) && $config['append_name'] != '') {
        if (strpos($user, '@') === false) {

            /* virtual host logic */
            if (strtolower($config['append_name']) == 'virtual_host' &&
               (isset($_SERVER['SERVER_NAME']) || isset($_SERVER['HTTP_HOST']))) {
                if (isset($config['use_server_name']) && $config['use_server_name'] == 
                    'true' && isset($_SERVER['SERVER_NAME'])) {
                    $shost = $_SERVER['SERVER_NAME'];
                }
                else {
                    $shost = $_SERVER['HTTP_HOST'];
                }
            
                /* use the defined list */
                if (isset($config['virtual_list']) && $config['virtual_list'] != '') {
                    if (strpos($config['virtual_list'], ',')) {
                        $hostpart = '('. 
                        str_replace(',', '|', $config['virtual_list']).')';
                        $domain = preg_replace("/^$hostpart\./", '', $shost);
                    }
                    else {
                        $hostpart = $config['virtual_list'];
                        $domain = preg_replace("/^$hostpart\./", '', $shost);
                    }
                }
                
                /* use our own logic */
                else {
                    if (!preg_match("/^(\d){1,3}\.(\d){1,3}\.(\d){1,3}\.(\d){1,3}$/",
                        $shost) && count(explode('.', $shost)) > 2) {
                        $domain = substr(strstr($shost, '.'), 1);
                    }
                    else {
                        $domain = $shost;
                    }
                }
                $user .= '@'.$domain;
            }

            /* static append name option */
            else {
                $user .= '@'.$config['append_name'];
            }
        }
        else {
            $domain = substr($user, (strpos($user, '@') + 1));
        }
    }
    else {
        if (!isset($config['domain']) || trim($config['domain']) == '') {
            if (isset($_SERVER['HTTP_HOST'])) {
                $domain = $_SERVER['HTTP_HOST'];
            }
        }
        else {
            $domain = $config['domain'];
        }
    }
    if (!$domain) {
        $domain = 'hastymail.org';
    }
    return array($user, $domain);
}

/**
 *  Determine the web browser capabilities for simple or normal mode.
 *
 *  @param  $string     string      the HTTP_USER_AGENT string from the browser
 *  @param  $palm       boolean     true if palm mode is disabled
 *  @return             boolean     true for simple mode and false for normal mode 
 *
 */
function hm_browser_type($string, $palm=false) {
    $simple = false;
    $simple_types = '(BlackBerry|Links|Lynx|Blazer|Nokia|UPG1|Elinks'.
                    '|PalmSource|PalmOS|WebPro|Netfront|Xiino)';
    $palm_types = '(BlackBerry|Blazer|Nokia|PalmSource|PalmOS|WebPro|Netfront|Xiino)';
    if (preg_match("/$simple_types/i", $string)) {
        $simple = true;
        if ($palm && preg_match("/$palm_types/i", $string)) {
            if (!isset($_SESSION['config']['no_palm_mode']) || 
                $_SESSION['config']['no_palm_mode'] != 'true') {
                $_SESSION['palm'] = 1;
            }
        }
    }
    elseif (preg_match("/^Mozilla\/(\d)/", $string, $matches)) {
        if (isset($matches[1]) && is_numeric($matches[1])) {
            if ($matches[1] < 3) {
                $simple = true;
            }
        }
    }
    return $simple;
}

/**
 *
 *  Our own htmlspecialchars function that does not care about charset
 *
 *  @param $string              string      the string to html-ize
 *  @return                     string      transformed text
 */
function hm_htmlize ($string) {
    if (preg_match("/(>|<|&(?!\S+;|\'))/", $string)) {
        $find    = array('&', '>', '<', "'");
        $replace = array('&amp;', '&gt;', '&lt;', '&#039;');
        $string =  str_replace($find, $replace, $string);
    }
    return $string;
}
 
/**
 *  clean and prepare text for display
 *
 *  @param  $text               string      the string to convert
 *  @param  $charset            string      the charset hastymail is using
 *  @param  $charset_convert    boolean     if the charset should be converted to utf-8
 *  @param  $code_page          array       the charset convertion table
 *  @param  $utf8               boolean     if the string is already utf-8
 *  @return                     string      cleaned text             
 *
 */
function hm_special_chars($text, $charset, $charset_convert, $code_page, $utf8=false) {
    $result = '';
    if ($charset_convert) {
        $result = hm_htmlize($text);
        $result = hm_utf8_convert($result, $code_page);
    }
    elseif ($charset == 'UTF-8') {
        if (!$utf8) {
            $text = utf8_encode($text);
        }
        $result = hm_utf8_to_html(hm_htmlize($text));
    }
    else {
        $result = hm_htmlize($text);
    }
    return $result;
}    

/**
 *  convert text to utf8
 *
 *  @param      $text       string  text to convert
 *  @param      $code_page  array   conversion table for the text charset 
 *  @return                 string  converted text
 *
*/ 
function hm_utf8_convert($text, $code_page) {
    $utf8='';
    $text_array = preg_split('//', $text, -1, PREG_SPLIT_NO_EMPTY);
    foreach ($text_array as $val) {
        if (ord($val) > 127) {
            $index = '0x'.strtoupper(dechex(ord($val)));
            $utf8 .= '&#x'.substr($code_page[$index], 2).';';
        }
        else {
            $utf8 .= $val;
        }
    }
    return $utf8;
}

/**
 *  split lines on word boudries
 *
 *  @param    $line       string      the string to be split
 *  @param    $limit      int         the strlen to split at
 *  @param    $hardbreak  boolean     true means split regardless of word boundries
 *  @param    $clean      boolean     false means don't mark split lines with "\"
 *  @reply    $reply      boolean     true to format for a reply
 *  @return                array      each line is one array element
 *
 */
function hm_split_line($line, $limit, $hardbreak=false, $clean=false, $reply=false) {
    if ($clean || (isset($_SESSION['settings']['wrap_clean']) && 
        $_SESSION['settings']['wrap_clean'] == 'true')) {
        $marker = '';
    }
    else {
        $marker = "     \\";
    }
    $quotemarker = '';
    if (strpos($line, '>') !== false && !$reply) {
        if (preg_match("/^(\s*>\s*){1,}/", $line, $regs)) {
            $quotemarker = $regs[0];
            $line = substr($line, strlen($quotemarker));
        }
    }
    $parts = array();
    $string = $line;
    while (strlen($string) > $limit) {
        if (!$hardbreak) {
            $chunk1 = substr($string, 0, ($limit-20));
            $chunk2 = substr($string, ($limit-20), 20);
            $split = strlen($chunk1) + (strlen($chunk2) - strlen(strrchr($chunk2, " ")));
            if ($split < $limit) {
                $parts[] = $quotemarker.substr($string, 0, $split).$marker; 
                $string = substr($string, $split+1);
            }
            else {
                break;
            }
        }
        else {
            $parts[] = $quotemarker.substr($string, 0, $limit).$marker;
            $string = substr($string, $limit);  
        }
    }
    $parts[] = $quotemarker.$string;
    return $parts; 
}

/**
 *  search struture array for a key, returning it's value
 *
 *  @param      $key        float       the key to search for 
 *  @param      $array      array       the structure array to search
 *  @param      $key_path   float       the parent if we know it 
 *  @param      $parent     boolean     return the parent value instead
 *  @return                 array       each element is a message part attribute
 *
 */
function &hm_in_multi_array ($key, $array, $key_path=array(), $parent=false) {
    $return = false;
    foreach ($array as $akey => $value) {
        if ($key == $akey) {
            if ($parent) {
                if (count($key_path) > 0) {
                    $return = array($key_path[count($key_path)-1][0], 
                                   $key_path[count($key_path)-1][1]);
                    break;
                }
                else {
                    $return = false;
                    break;
                }
            }
            else {
                $return = array($key, $value);
                break;
            }
        }
        if (isset($value['subs'])) {
            if (isset($value['subtype']) && $value['subtype'] == 'rfc822') {
                $key_path[] = array($akey, $value);
            }
            if($return = &hm_in_multi_array($key, $value['subs'], $key_path, $parent)) {
                break;
            }
        }
    }
    return $return;
}

/**
 *  search struture array for a cid: value and return message part number 
 *
 *  @param      $array      array       the structure array to search
 *  @param      $cid        string      the cid value to search for
 *  @return                 string      message part for the cid value
 *
 */
function hm_find_cid_part ($array, $cid) {
    $return = false;
    foreach ($array as $key => $subarray) {
        foreach ($subarray as $name => $val) {
            if ($name == 'id' && $val == $cid) {
                $return = $key;
                break;
            }
        }
        if ($return) {
            break;
        }
        if (isset($subarray['subs'])) {
            $return = hm_find_cid_part ($subarray['subs'], $cid);
        }
    }
    return $return;
}

/**
 *  decode specially encoded headers
 *
 *  @param  $string     string      value to decode
 *  @return             string      value converted to decoded text
 *
 */
function hm_decode_field($string) {
    $str = $_SESSION['str'];
    if (strlen($string) <= 1) {
        return $string;
    }
    elseif (strpos($string, ' ') === false ) {
        $parts[] = $string;
    }
    else {
        /* real encoded address from lkml. This violates the mime rfc afaict because
           there are no spaces or CRLFs after the encoded "words". nevertheless
           adding this preg_replace makes it work with the parser */

        //=?koi8-r?Q?=22?=Bansh=?koi8-r?Q?=22=20?= <bansh21@mail.ru>
        $string = preg_replace("/(=\?[^\?]+\?(q|b)\?[^\?]+\?=)/i", " \\1 ", $string);
        $parts = explode(' ', $string);
    }
    $return_string = '';
    foreach ($parts as $chunk) {
        $chunk = trim($chunk);
        if (strlen($chunk) < 5) {
            $return_string .= ' '.hm_htmlize($chunk);
        }
        else {
            $qq_start = '';
            $qq_stop = '';
            switch ($chunk{0}) {
                case '"':
                    $qq_start = '"';
                    $chunk    = substr($chunk, 1);
                    break;
                case "'":
                    $qq_start = "'";
                    $chunk    = substr($chunk, 1);
                    break;
                case '(':
                    $qq_start = '(';
                    $chunk    = substr($chunk, 1);
                    break;
                default:
                    break;
            }
            switch (substr($chunk, -1)) {
                case '"':
                    $qq_stop  = '"';
                    $chunk    = substr($chunk, 0, -1);
                    break;
                case "'":
                    $qq_stop  = "'";
                    $chunk    = substr($chunk, 0, -1);
                    break;
                case ')':
                    $qq_stop = ')';
                    $chunk    = substr($chunk, 0, -1);
                    break;
                default:
                    break;
            }
            $charset_convert = false;
            $code_page = false;
            $utf8 = false;
            if (substr($chunk, 0, 2) == '=?' && substr($chunk, -2) == '?=') {
                $chunk        = substr($chunk, 2);
                $charset      = strtolower(substr($chunk, 0, strcspn($chunk, '?')));
                $offset       = strlen($charset);
                $encoding     = substr($chunk, ($offset + 1), 1);
                $encoded_text = substr($chunk, ($offset + 3), -2);
                if (strtolower($encoding) == 'b') {
                    $chunk = base64_decode($encoded_text);
                }
                elseif (strtolower($encoding) == 'q') {
                    $chunk = &hm_quoted_decode($encoded_text);
                }
                else {
                    $chunk = $encoded_text;
                }
                if ($charset == 'utf-8') {
                    $utf8 = true;
                }
                if ($str['charset'] == 'UTF-8') {
                    /*if ($utf8) {
                        $chunk = utf8_encode($chunk);
                    }*/
                    if (isset($_SESSION['charsets'][$charset])) {
                        $code_page = $_SESSION['charsets'][$charset];
                        $charset_convert = true;
                    }
                }
            }
            $return_string .= ' '.$qq_start.hm_special_chars($chunk, $str['charset'], 
                              $charset_convert, $code_page, $utf8).$qq_stop;
        }
    }
    $return_string = trim($return_string);
    if (substr($return_string, 0, 2) == '=?' && substr($return_string, -2) == '?=') {
        $return_string = substr($return_string, 2, -2);
        $mark = strpos($return_string, '?');
        if ($mark !== false) {
            $return_string = substr($return_string, ($mark + 3));
        }
    }
    return $return_string; 
}

/**
 *  clean mailbox name
 *
 *  @param      $folders    array   the folders array
 *  @return                 array   each key is the folder, each value is the name
 *
 */
function hm_folder_dsp ($folders) {
    $folders_dsp = array();
    $root = false;
    if (isset($_SESSION['config']['allow_root_folders']) &&
        $_SESSION['config']['allow_root_folders'] == 'true') {
        $root = true;
    }
    $names = array();
    if (isset($_SESSION['settings']['namespace'])) {
        $names = $_SESSION['settings']['namespace'];
    }
    foreach($folders as $folder) {
        if ($folder != 'INBOX' && !in_array($folder, $names)) {
            if (isset($_SESSION['config']['prefix']) &&
                $_SESSION['config']['prefix'] != '' && 
                $_SESSION['config']['prefix'] != 'none' && empty($names)) {
                $folder_dsp = substr($folder, strlen($_SESSION['config']['prefix']));
            }
            elseif (preg_match("/^INBOX/", $folder) && !$root) {
                $folder_dsp = substr($folder, 6);
            }
            else {
                $folder_dsp = $folder;
            }
        }
        else {
            $folder_dsp = $folder;
        }
        $folders_dsp[$folder] = hm_translate_special_folders($folder_dsp);
    }
    return $folders_dsp;
}

/**
 *  clean address name for display
 *
 *  @param      $string     string      address value to clean up
 *  @param      $empty      string      value to display if the fields is empty
 *  @return                 string
 *
 */
function hm_address_dsp ($string, $empty=false) {
    if (trim($string) == '' && $empty) {
        return $empty;
    }
    if (strpos(trim($string), ' ') !== false) {
        $return_string = '';
        $string_parts = explode(' ', $string);
        foreach ($string_parts as $part) {
            switch (true) {
                case substr($part, 0, 1) == '"':
                    $return_string .= ' '.$part;
                    break;
                case strpos($part, '@'):
                case substr($part, 0, 1) == '<':
                    break;
                default:
                    $return_string .= ' '.$part;
                    break;
            }
        }
    }
    else {
        $return_string = $string;
    }
    $return_string = stripslashes(str_replace(array('(', ')', '&lt;', '&gt;', '"',
                     '&quot;'), '', $return_string));
    if (trim($string) == '' && $empty) {
        return $empty;
    }
    elseif (trim($return_string) == '') {
        $string = stripslashes(str_replace(array('(', ')', '&lt;', '&gt;', '"',
                         '&quot;'), '', $string));
        return $string;
    }
    else {
        return trim($return_string);
    }
}

/**
 *  clean up text/plain parts
 *
 *  @param  $text_part      string      the message text to format for display
 *  @param  $reply          boolean     when true format the text as a reply
 *  @param  $clean          boolean     don't add line break char (\)
 *  @param  $cutoff         int         line length to wrap to
 *  @return                 string      formatted text
 *
 */
function &hm_text_cleanup ($text_part, $reply=false, $clean=false, $cutoff=false) {
    $link_check = false;
    if (isset($_SESSION['settings']['click_links']) && 
        $_SESSION['settings']['click_links'] == 'true') {
        $link_check = true;
    }
    $result = array();
    if ($reply) {
        $clean = true;
        $def_divider = "\n> ";
        $prefix = "> ";
        if (!$cutoff) {
            $cutoff = 72;
        }
    }
    else {
        $def_divider = "\n";
        $prefix = '';
        if (!$cutoff) {
            if (isset($_SESSION['settings']['read_wrap'])) {
                $cutoff = $_SESSION['settings']['read_wrap'];
            }
            else {
                $cutoff = 90;
            }
        }
    }
    $text_part = preg_replace("/\t/", '    ', $text_part);
    $lines = preg_split("/\n/", $text_part);
    $carry_over = '';
    $add_blank_line = false;
    foreach ($lines as $index => $line) {
        if ($carry_over != '' && trim($line) == '' && $reply) {
            if ($divider != $def_divider) {
                $line = ltrim($divider).$carry_over;
            }
            else {
                $line = trim($divider).' '.ltrim($carry_over);
            }
            $add_blank_line = true;
        }
        else {
            if ($reply) {
                if (strpos($line, '>') !== false) {
                    if (preg_match("/^((\s*>\s*){1,})(.*)/", $line, $regs)) {
                        $line = rtrim($regs[1], "\r\n").' '.ltrim($carry_over, ' ').
                                $regs[3];
                    }
                    else {
                        $line = ltrim($carry_over, ' ').$line;
                    }
                }
                else {
                    $line = ltrim($carry_over, ' ').$line;
                }
            }
            $line = $prefix.$line;
            $divider = $def_divider;
            if (strpos($line, '>') === true) {
                if (preg_match("/^(\s*>\s*){1,}/", $line, $regs) && $reply) {
                    $divider = "\n".rtrim($regs[0], ' ')." ";
                }
            }
        }
        if (strlen($line) > $cutoff) {
            $link = false;
            $regex = "/^((http|ftp|rtsp)s?:\/\/(%[[:digit:]A-Fa-f][[:digit:]A-Fa-f]|".
                     "[-_\.!~\*';\/\?#:@&=\+$,[:alnum:]])+)/i";
            if ($link_check && preg_match($regex, $line, $matches) && !$reply) {
                $link = $matches[1]."\n";
                $line = substr($line, 0, (0 - strlen($matches[1])));
            }
                $temp = hm_split_line($line, $cutoff, false, $clean, $reply);
                if (count($temp) == 1) {
                    $result[] = rtrim(implode($divider, 
                                hm_split_line($line, $cutoff, true, $clean, $reply))).
                                "\n";
                    $carry_over = '';
                }
                elseif(strlen($temp[(count($temp) - 1)]) > $cutoff) {
                    $result[] = rtrim(implode($divider, 
                                array_slice($temp, 0, (count($temp)-1))))."\n";
                    $result[] = rtrim(implode($divider, hm_split_line(
                                $temp[(count($temp)-1)], $cutoff, true, $clean, $reply))).
                                "\n";
                    $carry_over = '';
                }
                else {
                    if ($reply) {
                        $carry_over = ' '.rtrim($temp[(count($temp)-1)]).' ';
                        array_pop($temp);
                    }
                    else {
                        $carry_over = '';
                    }
                    $result[] = rtrim(implode($divider, $temp))."\n";
                }
            if ($link) {
                $result[] = $link;
            }
        }
        else {
            $result[] = rtrim($line)."\n";
            if ($add_blank_line) {
                $result[] = ">\n";
                $add_blank_line = false;
            } 
            $carry_over = '';
        }
    }
    $result[] = rtrim($carry_over)."\n";
    while (isset($result[(count($result)-1)]) && 
           preg_match("/^\s*>*\s*>*\n$/", $result[(count($result)-1)])) {
        array_pop($result);
    }
    $body = "\n".implode("", $result)."\n";
    return $body;
}

/**
 *  decode quoted printable values
 *
 *  @param      $string     string      string to decode
 *  @return                 string      decoded text
 *
 */
function &hm_quoted_decode($string) {
    $string = preg_replace("/(\=[0-9A-Za-z]{2})\n/", "\\1 \n", $string);
    $result = preg_replace("/\=([0-9A-Za-z]{2})/e", 
              "''.chr(hexdec('\\1')).''", $string);
    $result = preg_replace("/\=\n/m", '', $result);
    return $result;
}

/**
 *  find and return parts of a message
 *
 *  @param      $imap_stream    resource    connection to imap
 *  @param      $structure      array       message mime structure
 *  @param      $uid            int         message uid
 *  @param      $part           float       part to return
 *  @param      $reply          boolean     format as a reply
 *  @param      $clean          boolean     don't add line break char (\)
 *  @param      $cutoff         boolean     flag to break LARGE messages
 *  @returns                    string      formatted message part
 *
 */
function &hm_find_display_part ($imap_stream, $structure, $uid, $part=false, 
                               $reply=false, $clean=false, $cutoff=false) {
    $str = $_SESSION['str'];
    $result_fallback = false;
    if (!$part) {
        /* here we don't know the text part we want so find the first one */
        if (is_array($structure)) {
            foreach($structure as $index => $array) {
                if ($array['type'].$array['subtype'] == 'textplain' ||
                    $array['type'].$array['subtype'] == 'textenriched') {
                    $text_part = &hm_get_message_part($imap_stream, $uid, $index,
                                 $cutoff);
                    if (isset($array['charset'])) {
                        $chr = $array['charset'];
                    }
                    else {
                        $chr = false;
                    }
                    if ($array['encoding'] == 'quoted-printable') {
                        $text_part = &hm_quoted_decode($text_part);
                    }
                    elseif ($array['encoding'] == 'base64') {
                        $text_part = base64_decode($text_part);
                    }
                    $text_part = &hm_text_cleanup($text_part, $reply, $clean);
                    if ($array['type'].$array['subtype'] == 'textenriched') {
                        if ($reply) {
                            $text_part = str_replace("\n>\n>", "\n>", $text_part);
                        }
                        else {
                            $text_part = str_replace("\n\n", "\n", $text_part);
                        }
                    }
                    if (trim($text_part) != '') {
                        $result = array($index, $text_part, array($array['encoding'], 
                                        'text', 'plain', 'textplain', $chr));
                        return $result;
                    }
                    else {
                        $result_fallback = array($index, $text_part,
                                           array($array['encoding'], 'text', 'plain',
                                           'textplain', $chr));
                    }
                }
                if (isset($array['subs'])) {
                    $result = &hm_find_display_part ($imap_stream, $array['subs'], $uid, 
                                                    false, $reply, $clean, $cutoff);
                    return $result;
                }
            }
        }
    }
    else {
        /* this is for parts selected from the message parts links in message.php */
        $dsp_type = false;
        $types = array(
        'text:plain'  => array ('textplain', 'messagedisposition-notification',
                                'messagedelivery-status', 'textenriched', 
                                'textrfc822-headers', 'messagerfc822-headers',
                                'applicationpgp-signature'),
        'image:jpeg'  => array('imagejpeg', 'imagepjpeg', 'imagejpg'),
        'image:png'   => array('imagepng'),
        'image:gif'   => array('imagegif'),
        'text:html'   => array('texthtml'));

        $attributes = &hm_in_multi_array ($part, $structure);
        if (isset($attributes[1]['type'])) {
            $type = $attributes[1]['type'];
        }
        else {
            $type = false;
        }
        if (isset($attributes[1]['name'])) {
            $name = $attributes[1]['name'];
        }
        else {
            $name = false;
        }
        if (isset($attributes[1]['subtype'])) {
            $subtype = $attributes[1]['subtype'];
        }
        else {
            $subtype = false;
        }
        if (isset($attributes[1]['encoding'])) {
            $decode = $attributes[1]['encoding'];
        }
        else {
            $decode = false;
        }
        if (isset($attributes[1]['charset'])) {
           $chr = $attributes[1]['charset'];
        }
        else {
            $chr = false;
        }
        $dsp_type = $type.$subtype;
        $dsp_flag = false;
        foreach ($types as $key => $array) {
            if (in_array($dsp_type, $array)) {
                $dsp_type = str_replace(':', '', $key);
                $dsp_flag = true;
                break;
            }
        }
        if (!$dsp_flag) {
            if ($type == 'text') {
                $dsp_type = 'textplain';
            }
            if ($type == 'image') {
                $dsp_type = 'image'.$subtype;
            }
        }
        $message_part = &hm_get_message_part($imap_stream, $uid, $part, $cutoff);
        if ($decode == 'quoted-printable') {
            $message_part = &hm_quoted_decode($message_part); 
        }
        elseif ($decode == 'base64') {
            $message_part = base64_decode($message_part);
        }
        if ($dsp_type == 'textplain') {
            $message_part = str_replace("\0", '', $message_part);
            if (!$clean) {
                $message_part = &hm_text_cleanup($message_part);
            }
            if ($type.$subtype == 'textenriched') {
                $message_part = str_replace("\n\n", "\n", $message_part);
            }
        }
        $result = array($part, $message_part, 
                        array( $decode, $type, $subtype, $dsp_type, $chr, $name));
        return $result;
    }
    /* if we could not find a text part return a message */
    if (!$result_fallback) {
        $text_part = "\n\n     ".$str[194]."\n\n";
        $result = array('0', $text_part, array('none', 'text', 'plain', 'textplain'));
        return $result;
    }
    else {
        return $result_fallback;
    }
} 

/**
 *  check the cahce and decide on sort update 
 *
 *  @param  $mailbox    string      folder to check cahce status on
 *  @return             boolean     if the cache can be re-used
 *
 */
function hm_check_cache($mailbox) {
    $cache = false;
    if (isset($_SESSION['sort_data'][$mailbox])) {
        if (!isset($_SESSION['changed'][$mailbox])) {
            $cache = true;
        }
        else {
            unset($_SESSION['changed'][$mailbox]);
        }
    }
    return $cache;
} 

/**
 *  parse one set of message headers
 *
 *  @param  $header_array   array   the message headers
 *  @return                 array   keys are field names
 *  
 */
function hm_parse_header($header_array) {
    $result = array();
    $n = 1;
    $count = count($header_array);
    for ($i=0;$i<$count;$i++) {
        if (preg_match("/^(\S+:)(.*)/i", $header_array[$i], $regs)) {
            $field = $regs[1];
            $value = trim($regs[2]);
            if (isset($header_array[$i+1]) && 
                      preg_match("/^(\s|\t)/", $header_array[$i+1])) {
                $k = $i+1;
                while (preg_match("/^(\s|\t)/", $header_array[$k])) {
                    $value .= " ".trim($header_array[$k]);
                    if (isset($header_array[$k+1])) {
                        $k++;
                    }
                    else {
                        break;
                    }
                }
            }
            if (isset($result[strtolower(rtrim($field, ":"))])) {
                $key = strtolower(rtrim($field, ":")).'_'.$n;
                if ($key == 'subject' || $key == 'from' || 
                    $key == 'to' || $key == 'cc' || $key == 'reply-to') {
                    $value = hm_decode_field($value);
                }
                else {
                    $value = hm_htmlize($value);
                }
                $result[$key] = $value;
                $n++;
            }
            else {
                $key = strtolower(rtrim($field, ":"));
                if ($key == 'subject' || $key == 'from' || 
                    $key == 'to' || $key == 'cc' || $key == 'reply-to') {
                    $value = hm_decode_field($value);
                }
                else {
                    $value = hm_htmlize($value);
                }
                $result[$key] = $value;
            }
        }
    }
    return $result;
}

/**
 *  simple error check for IMAP responses
 *
 *  @param  $string     string  imap response to check
 *  @param  $prefix     string  the imap command prefix (like A00001)
 *  @param  $command    string  the command that was sent to the imap server
 *  @param  $inline     boolean produce errors with inline HTML 
 *  @return             void    exits calling script for a read error
 */
function hm_check_errors($string, $prefix, $command, $inline=false){
    $str = $_SESSION['str'];
    if (strpos($string, $prefix) === false) {
        $settings = $_SESSION['settings'];
        if ($inline) {
            $_SESSION['notices'][] = $str[193]; 
            $_SESSION['notices'][] = $str[189]; 
            $_SESSION['notices'][] = $str[190].':'; 
            $_SESSION['notices'][] = hm_htmlize($command); 
            $_SESSION['notices'][] = $str[191].':'; 
            $_SESSION['notices'][] = hm_htmlize($string); 
        }
        else {
            $simple = false;
            if ((isset($_SESSION['settings']['simple']) && 
                $_SESSION['settings']['simple'] == 'true') || (isset(
                $_SESSION['auto_simplify']) && $_SESSION['auto_simplify'] == 'true')) {
                $simple = true;
            }
            if (isset($config['default_theme']) && $config['default_theme'] != '') {
                $style = str_replace(array('_large', '_small'), '', 
                $config['default_theme']);
            }
            else {
                $style = 'default.css';
            }
            $page_data['title'] = 'Hastymail'.' '.$str[193];
            $page_data['lbl_heading'] = $str[193]; 
            $page_data['lbl_heading2'] = $str[189]; 
            $page_data['lbl_command']= $str[190].':'; 
            $page_data['command'] = hm_htmlize($command); 
            $page_data['lbl_result'] = $str[191].':'; 
            $page_data['result'] = hm_htmlize($string); 
            $page_data['charset'] = $str['charset'];
            $page_data['style'] = $style;
            $page_data['lbl_link'] = $str[192];
            if (isset($_SESSION['settings']['template_set'])) {
                $template_set = $_SESSION['settings']['template_set'];
            }
            elseif (isset($_SESSION['config']['template_set']) &&
                $_SESSION['config']['template_set'] != '') {
                $template_set = $_SESSION['config']['template_set'];
            }
            else {
                $template_set = 'default';
            }
            if ($simple) {
                hm_output_page($page_data, 'imaperror_template.php', 'simple');
            }
            else {
                hm_output_page($page_data, 'imaperror_template.php', $template_set);
            }
            exit;
       }
    }
}

/**
 *  takes the result of a message action and issue the correct imap
 *  commands, then returns the error/success information 
 *
 *  @param      $action         string      the imap action to perform
 *  @param      $imap_stream    resource    the connection to imap
 *  @param      $uid_string     string      message uids to operate on
 *  @param      $destination    string      if move or copy then this is target folder
 *  @param      $large          boolean     flag for really large message counts
 *  @param      $sort_type      string      sort method 
 *  @param      $mailbox        string      the currently selected mailbox
 *  @param      $dsp_count      int         # of messages we are dealing with
 *  @param      $dsp_end        string      end of the response string we send back
 *  @param      $skip           boolean     skip moving to trash when true
 *  @return                     string      sort type
 *
 */
function hm_message_controls ($action, $imap_stream, $uid_string, $destination,
                              $large, $sort_type, $mailbox, $dsp_count, $dsp_end,
                              $skip=false) {
    $sort = false;
    $str = $_SESSION['str'];
    $dsp_end2 = $dsp_end;
    if (isset($_SESSION['config']['utf7_decode'])) {
        $dsp_end .= '<small>&nbsp;&nbsp;&nbsp;['.
                     hm_htmlize(utf8_encode(imap_utf7_decode($mailbox))).
                     ']</small>';
        $dsp_end2 .= '<small>&nbsp;&nbsp;&nbsp;['.
                     hm_htmlize(utf8_encode(imap_utf7_decode($mailbox))).
                     ' -> '.hm_htmlize(utf8_encode(imap_utf7_decode( $destination))).
                     ']</small>';
    } 
    else {
        $dsp_end .= '<small>&nbsp;&nbsp;&nbsp;['.
                    hm_htmlize(urldecode($mailbox)).']</small>';
        $dsp_end2 .= '<small>&nbsp;&nbsp;&nbsp;['.hm_htmlize($mailbox).
                     ' -> '.hm_htmlize($destination).']</small>';
    }
    switch($action) {
        case 'move':
            if ($err = hm_imap_action($imap_stream, $uid_string, 'COPY', 
                                    $destination, $large)) {
                $_SESSION['notices'][] = $err;
            }
            elseif ($err = hm_imap_action($imap_stream, $uid_string, 'DELETE', 
                                          false, $large)) {
                $_SESSION['notices'][] = $err;
            }
            elseif ($err = hm_imap_action($imap_stream, $uid_string, 'EXPUNGE',
                                          false, $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[186]." $dsp_count $dsp_end2";
            }
            break;
        case 'copy':
            if ($err = hm_imap_action($imap_stream, $uid_string, 'COPY', 
                                      $destination, $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[187]." $dsp_count $dsp_end2";
            }
            break;
        case 'expunge':
            $action_dsp = '';
            if ($err = hm_imap_action ($imap_stream, $uid_string, 'EXPUNGE',
                                           false, $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[364];
            }
            break;
        case 'delete':
            $action_dsp = '';
            if (isset($_SESSION['settings']['trash_folder']) &&
                $_SESSION['settings']['trash_folder'] != $mailbox && !$skip) {
                if ($err = hm_imap_action($imap_stream, $uid_string, 
                                                       'READ', false, $large)) {
                    $action_dsp = $err;
                }
                elseif ($err = hm_imap_action ($imap_stream, $uid_string, 'COPY',
                                           $_SESSION['settings']['trash_folder'],
                                           $large)) {
                    $action_dsp = $str[27];
                    unset($_SESSION['settings']['trash_folder']);
                    hmw_write_settings($_SESSION['user'], $config); 
                }
            }
            if ($action_dsp == '') {
                if ($err = hm_imap_action($imap_stream, $uid_string, 'DELETE', 
                                          false, $large)) {
                    $_SESSION['notices'][] = $err;
                }
                elseif ($err = hm_imap_action ($imap_stream, $uid_string, 'EXPUNGE',
                                               false, $large)) {
                    $_SESSION['notices'][] = $err;
                }
                else {
                    $_SESSION['notices'][] = $str[89]." $dsp_count $dsp_end";
                }
            }
            else {
                $_SESSION['notices'][] = $action_dsp;
            }
            break;
        case 'read':
            if ($err = hm_imap_action($imap_stream, $uid_string, 'READ', false, $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[2]." $dsp_count $dsp_end";
            }
            break;
        case 'unread':
            if ($err = hm_imap_action($imap_stream, $uid_string, 'UNREAD',
                                      false, $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[3]." $dsp_count $dsp_end";
            }
            break;
        case 'flag':
            if ($err = hm_imap_action($imap_stream, $uid_string, 'FLAG', false, $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[327]." $dsp_count $dsp_end";
            }
            break;
        case 'unflag':
            if ($err = hm_imap_action($imap_stream, $uid_string, 'UNFLAG', false, 
                $large)) {
                $_SESSION['notices'][] = $err;
            }
            else {
                $_SESSION['notices'][] = $str[328]." $dsp_count $dsp_end";
            }
            break;
        case 'sort';
            $sort = $sort_type;
            $_SESSION['notices'][] = $str[188];
            break;
        default:
            break;
    }
    return $sort;
}

/**
 *  looks for message attachments and writes the entire message
 *  to the supplied stream. Used for both IMAP and SMTP
 *
 *  @param  $stream     resource    the connection to the server
 *  @param  $body       string      the first part of the message (headers and body)
 *  @param  $bndry      string      mime boundary
 *  @return             boolean     true on success, false on failure
 *
 */
function hm_write_data($stream, $body, $bndry) {
    $size = 0;
    if (is_resource($stream)) {
        fputs($stream, $body);
        $size = strlen($body);
        foreach ($_SESSION['attachments'] as $name => $vals) {
            clearstatcache();
            if (file_exists($_SESSION['config']['settings_dir'].'attachments/'.$name)
                && is_resource($stream)) {
                fputs($stream, "\r\n$bndry\r\n");
                fputs($stream, $vals['header']);
                $size += strlen("\r\n$bndry\r\n".$vals['header']);
                $fh = fopen($_SESSION['config']['settings_dir'].'attachments/'.$name,'r');
                if (is_resource($fh)) {
                    while (!feof($fh)) {
                        $tmp = fread($fh, 1024);
                        fputs($stream, $tmp);
                        $size += strlen($tmp);
                    }
                    fclose($fh);
                }
                //fputs($stream, "\r\n");
                //$size += 2;
            }
            else {
                return false;
            }
        }
    }
    else {
        return false;
    }
    fputs($stream, "\r\n$bndry--");
    $size += strlen("\r\n$bndry--");
    return $size; 
}

/**
 *
 * convert raw sizes into human readable form
 *
 *  @param      $rawsize    int     size in bytes
 *  @return     $size       string  human readable size
 *
 */
function hm_size_format ($rawsize) {
    $size = 'unkown';
    if ($rawsize > 1024 && $rawsize < 1048576) {
        $size = round(($rawsize/1024), 1).' Kb';
    }
    elseif ($rawsize > 1048576) {
        $size = round(($rawsize/1048576), 1).' Mb';
    }
    elseif ($rawsize < 1024) {
        $size = $rawsize.' b';
    }
    return $size;
}

/**
 *
 * Simple sort function for the search and new pages 
 *
 *  @param      $a          array     size in bytes
 *  @param      $b          array     size in bytes
 *  @return                 boolean   
 *
 */
function hm_date_cmp($a, $b) {
    if ($a[8] == $b[8]) {
        return 1;
    }
    elseif ($a[8] < $b[8]) {
        return 1;
    }
    else {
        return -1;
    }
}

/**
 *
 * debug function to spit out time data
 *
 */
function hm_time() {
    $time = microtime();
    $seconds = substr($time, -2);
    $parts = explode(' ', $time);
    $micro = substr($parts[0], 2, 5);
    return $seconds.'.'.$micro;
}
?>
