<?php

/*  wrapper_functions.php: wrappers for various functions in lib/
*
    Copyright (C) 2002,2003,2004  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: wrapper_functions.php,v 1.39 2005/05/16 16:37:15 sailfrog Exp $
*/

/* Include the needed files for the calling script */
require('constant.php');
require('session.php');
switch (basename($_SERVER['PHP_SELF'])) {
    case 'thread_view.php':
    case 'new.php':
    case 'mailbox.php':
        require('session_2.php');
        require('general.php');
        require('imap.php');
        require('imap_2.php');
        require('html_functions.php');
        break;
    case 'print_view.php':
    case 'message.php':
        require('session_2.php');
        require('general.php');
        require('imap.php');
        require('imap_2.php');
        require('bodystructure.php');
        require('html_functions.php');
        require('html_functions_2.php');
        break;
    case 'search.php':
        require('general.php');
        require('imap.php');
        require('imap_2.php');
        require('session_2.php');
        require('html_functions.php');
        break;
    case 'logout.php':
        require('session_2.php');
        require('general.php');
        break;
    case 'folders.php':
        require('general.php');
        require('imap.php');
        require('imap_2.php');
        require('imap_3.php');
        require('session_2.php');
        require('html_functions.php');
        break;
    case 'contacts.php':
    case 'options.php':
        require('../lang/lang_index.php');
        require('general.php');
        require('imap.php');
        require('imap_2.php');
        require('session_2.php');
        require('html_functions.php');
        break;
    case 'about.php':
        require('general.php');
        require('imap.php');
        require('html_functions.php');
        break;
    case 'iframe.php':
        break; 
    case 'compose.php':
        require('general.php');
        require('imap.php');
        require('imap_2.php');
        require('imap_3.php');
        require('session_2.php');
        require('smtp.php');
        require('html_functions.php');
        require('bodystructure.php');
        break;
    case 'download.php':
        require('general.php');
        require('imap.php');
        require('imap_3.php');
        require('bodystructure.php');
        break;
    default:
        hm_logged_out($hm_input['id']);
        break;
}

/**
 *  Wrapper function to write contact information
 * 
 *  @param      string      $user       username
 *  @param      array       $config     configuration settings
 *  @param      array       $contacts   array of user contacts
 *  @return     boolean                 Tru one success, false on failure
 */
function hmw_write_contacts($user, $config, $contacts=array()) {
    $result = false;
    if (isset($config['contacts_storage']) && $config['contacts_storage'] != '') {
        $storage = $config['contacts_storage'];
    }
    else {
        $storage = 'file';
    }
    switch ($storage) {
        case 'db':
            $result = hm_write_db_contacts($user, $config, $contacts);
            break;
        default:
            $result = hm_write_contacts($user, $config['settings_dir'], $contacts);
            break;
    }
    return $result;
}

/**
 *  wrapper function to pull in contact lists from different sources.
 *  @param      $user       string      username
 *  @param      $config     string      user settings directory
 *  @param      $key        string      encryption key
 *  @param      $search     string      search filter
 *  @param      $ldap       boolean     flag to disable ldap contacts
 *  @return                 array       array of user contacts
 *
 */
function hmw_get_contacts($user, $config, $key=false, $search=false, $ldap=false) {
    $contacts = array();
    if ($ldap && isset($config['ldap_server']) && $config['ldap_server'] != '') {
        if (!isset($config['ldap_search_only']) ||
            $config['ldap_search_only'] == 'false' || $search) {
            $contacts = array_merge($contacts, 
                        hm_get_ldap_contacts($config, $key, $search));
        }
    }

    if (!empty($config['contacts_storage']) && ($config['contacts_storage'] == 'db')) {
        $contacts = array_merge($contacts, hm_get_db_contacts($user, $config, $search));
    } else {
        $contacts = array_merge($contacts, hm_get_contacts($user, $config['settings_dir'], $search));
    }

    $contacts = hm_sort_contacts($contacts);
    return $contacts;
}


/**
 *  wrapper around get_sort_info().
 *
 *  @param  $imap_stream    resource    connection to imap
 *  @param  $sort_type      string      sort method
 *  @param  $mailbox        string      folder to sort
 *  @param  $quick          boolean     quick flag for thread view
 *  @param  $filter         string      sort filter
 *  @return                 array       sort information array
 *
 */
function &hmw_get_sort_info ($imap_stream, $sort_type, $mailbox, $quick=false, 
                             $filter=false) {
    $sort_info = '';
    if (isset($_SESSION['config']['client_sort']) && 
              $_SESSION['config']['client_sort'] == 'true') {
        $sort_info = &hm_client_side_sort($imap_stream, $mailbox, $sort_type, $filter);
    }
    else {
        $sort_info = &hm_get_sort_info($imap_stream, $sort_type, $mailbox, $quick,
                                       $filter);
    }
    return $sort_info;
}

/**
 *  wrapper around hm_crypt()
 *
 *  @param  $string     string      string to encrypt/decrypt
 *  @param  $config     array       configuration values
 *  @param  $type       string      flag to decrypt or encrypt string
 *  @param  $key        string      encryption key
 *  @return             string      encrypted/decrypted string
 *
 */
function hmw_crypt($string, $config, $type, $key) {
    if (isset($config['use_mcrypt']) && $config['use_mcrypt'] == 'true') {
        if (isset($config['mcrypt_cipher'])) {
            $cipher = strtolower($config['mcrypt_cipher']);
        }
        else {
            $cipher = 'des';
        }
        return hm_mcrypt($string, $type, $cipher, $key);
    }
    return hm_crypt($string, $key);
}

/**
 * wrapper around hm_get_settings()
 *
 * @param   string  $user       username
 * @param   array   $config     configuration values
 * @return  array               user settings array
 */
function hmw_get_settings($user, $config) {
    $settings = array();
    if (!empty($config['settings_storage'])) {
        $setstore = strtolower($config['settings_storage']);
    }
    else {
        $setstore = 'file';
    }
    switch($setstore) {
        case 'db':
            $settings = hm_get_db_settings($user, $config);
            break;
        case 'file':
        default:
            $settings = hm_get_file_settings($user, $config);
    }
    $settings = hm_get_settings($settings);
    return $settings;
}

/**
 * wrapper around hm_write_settings()
 *
 * @param   string  $user       username
 * @param   array   $config     configuration values
 * @return  void
 */
function hmw_write_settings($user, $config) {

    if (!empty($config['settings_storage'])) {
        $setstore = strtolower($config['settings_storage']);
    } else {
        $setstore = 'file';
    }

    switch($setstore) {
        case 'db':
            $settings = hm_write_db_settings($user, $config);
            break;
        case 'file':
        default:
            $settings = hm_write_file_settings($user, $config['settings_dir']);
    }

}


?>
